TrainerFu = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0", "FuBarPlugin-2.0", "AceModuleCore-2.0")

local tablet = AceLibrary("Tablet-2.0")
local abacus = AceLibrary("Abacus-2.0")
local crayon = AceLibrary("Crayon-2.0")
local L = AceLibrary("AceLocale-2.2"):new("FuBar_TrainerFu")

TrainerFu:RegisterDB("TrainerFuDB", "TrainerFuDBPerChar")
TrainerFu:RegisterDefaults('profile', {
	shownotlearnable = false,
	showlearnablenext = true,
	showskillcost = true,
	showlevelreq = true,
	showingtrainernames = true,
	showtrainablequests = true,
	showtotaltrainingcost = true,
	skillinfoframepoint = { point = "CENTER", x = 0, y = 0},
})
TrainerFu:RegisterDefaults('char', {
	trainers = {},
	trainerTypes = {},
	skillDB = {}
})

TrainerFu.hasIcon = "Interface\\AddOns\\FuBar_TrainerFu\\icons\\Misc"
TrainerFu.independantProfile = true
TrainerFu.clickableTooltip = true

local optionsTable = {
	handler = TrainerFu,
	type = 'group',
	args = {
		show = {
			type = 'group',
			name = L["Show"],
			desc = L["Show"],
			args = {
				shownotlearnable = {
					type = 'toggle',
					name = L["All skills"],
					desc = L["All skills including those you can't learn yet"],
					get = "IsShowingNotLearnable",
					set = "ToggleShowNotLearnable",
				},
				showlearnablenext = {
					type = 'toggle',
					name = L["Learnable next"],
					desc = L["Show the skills you can learn when you hit the next equal level"],
					get = "IsShowingLearnableNext",
					set = "ToggleShowingLearnableNext",
				},
				showskillcost = {
					type = 'toggle',
					name = L["Skill training cost"],
					desc = L["Show how much learning a skill cost"],
					get = "IsShowingSkillCost",
					set = "ToggleShowingSkillCost",
				},
				showlevelreq = {
					type = 'toggle',
					name = L["Skill level requirement"],
					desc = L["Show required level to learn skill"],
					get = "IsShowingSkillLevelReq",
					set = "ToggleShowingSkillLevelReq",
				},
				showtrainernames = {
					type = 'toggle',
					name = L["Trainers name and location"],
					desc = L["Show all known trainers and their location"],
					get = "IsShowingTrainerNames",
					set = "ToggleShowingTrainerNames",
				}
			}
		},
		showtext = {
			type = 'group',
			name = L["FuBar Text"],
			desc = L["Text shown in FuBar"],
			args = {
				showtrainablequests = {
					type = 'toggle',
					name = L["Show trainable quests"],
					desc = L["Show number of trainable quests in FuBar."],
					get = "IsShowTrainableSkills",
					set = "ToggleShowTrainableQuests",
				},
				showtotaltrainingcost = {
					type = 'toggle',
					name = L["Show training cost"],
					desc = L["Show total training cost in FuBar."],
					get = "IsShowTrainingCost",
					set = "ToggleShowTrainingCost",
				}
			}
		}
	}
}

TrainerFu:RegisterChatCommand({ L["/trainerfu"] }, optionsTable)
TrainerFu.OnMenuRequest = optionsTable

--------------------------------
-- TOGGLE FUNCTIONS
--------------------------------
function TrainerFu:CanLearnSkillNext(skill)
	--Can skill be learned at next equal level?
	return self:CanLearnSkill(skill, 2)
end

function TrainerFu:ToggleShowNotLearnable()
	self.db.profile.showskillsnotlearnable = not self.db.profile.showskillsnotlearnable
	self:Update()
end
function TrainerFu:IsShowingNotLearnable()
	return self.db.profile.showskillsnotlearnable
end

function TrainerFu:ToggleShowingLearnableNext()
	self.db.profile.showinglearnablenext = not self.db.profile.showinglearnablenext
end
function TrainerFu:IsShowingLearnableNext()
	return self.db.profile.showinglearnablenext
end

function TrainerFu:ToggleShowingSkillCost()
	self.db.profile.showingskillcost = not self.db.profile.showingskillcost
end
function TrainerFu:IsShowingSkillCost()
	return self.db.profile.showingskillcost
end

function TrainerFu:ToggleShowingSkillCost()
	self.db.profile.showingskillcost = not self.db.profile.showingskillcost
end
function TrainerFu:IsShowingSkillCost()
	return self.db.profile.showingskillcost
end

function TrainerFu:ToggleShowingSkillLevelReq()
	self.db.profile.showingskilllevelreq = not self.db.profile.showingskilllevelreq
end
function TrainerFu:IsShowingSkillLevelReq()
	return self.db.profile.showingskilllevelreq
end

function TrainerFu:ToggleShowingTrainerNames()
	self.db.profile.showingtrainernames = not self.db.profile.showingtrainernames
end
function TrainerFu:IsShowingTrainerNames()
	return self.db.profile.showingtrainernames
end

--FuBar text
function TrainerFu:ToggleShowTrainableQuests()
	self.db.profile.showtrainablequests = not self.db.profile.showtrainablequests
	self:Update()
end
function TrainerFu:IsShowTrainableSkills()
	return self.db.profile.showtrainablequests
end

function TrainerFu:OnInitialize()

end

function TrainerFu:OnEnable()
	self.characterName = UnitName("player")
	self.characterLevel = UnitLevel("player")
	
	self:RegisterEvent("PLAYER_LEVEL_UP", 2)
	self:RegisterEvent("SKILL_LINES_CHANGED", 2)
	self:RegisterEvent("TrainerFu_SkillsUpdated", "UpdateSkillInformation")
end

-------------
--EVENTS
-------------
function TrainerFu:PLAYER_LEVEL_UP(newLevel)
	--Set new character level
	self.characterLevel = newLevel
	self:TriggerEvent("TrainerFu_SkillsUpdated")
end

function TrainerFu:SKILL_LINES_CHANGED()
	self:UpdateSkillRanks()
	self:TriggerEvent("TrainerFu_SkillsUpdated")
end

function TrainerFu:UpdateSkillInformation()
	self:UpdateNumberofLearnableSkills()
	self:Update()
end

function TrainerFu:UpdateSkillRanks()
	self.skillRanks = {}
	local numSkills = GetNumSkillLines();
	for i=1, numSkills do
		local skillName, isHeader, isExpanded, skillRank, numTempPoints, skillModifier, skillMaxRank, 
			isAbandonable, stepCost, rankCost, minLevel, skillCostType, skillDescription = GetSkillLineInfo(i);
		--Only save important skills, omitt unimportant stuff like languages and that you can wear cloth armor
		if not isHeader and skillMaxRank ~= 1 and not string.find(skillName, "Language") then
			self.skillRanks[skillName] = {}
			self.skillRanks[skillName].rank = skillRank
			self.skillRanks[skillName].maxRank = skillMaxRank
		end
	end
end

function TrainerFu:GetTrainingCost()
	return abacus:FormatMoneyFull(self.costOfLearnableSkills, true, coloured)
end

function TrainerFu:GetCoordinateText(trainer)
	if trainer.xcoord ~= 0 and trainer.ycoord ~= 0 then
		return string.format(" (%.0f, %.0f)", trainer.xcoord * 100, trainer.ycoord * 100)
	else return "" end
end

function TrainerFu:GetNumberOfLearnableSkills()
	if self.numberOfLearnambleSkills > 0 then return "|cff00FF00"..self.numberOfLearnambleSkills.."|r"
	else return "|cffff0000"..self.numberOfLearnambleSkills.."|r" end
end

function TrainerFu:SkillReqIsMet(skill)
	if skill.skillrankreq and self.skillRanks[skill.skillreq] and skill.skillrankreq > self.skillRanks[skill.skillreq].rank then
		return false
	elseif skill.skillrankreq and not self.skillRanks[skill.skillreq] then
		return false
	else return true end
end

function TrainerFu:SkillAbilityReqIsMet(skill)
	if skill.abilityreq and not skill.hasabilityreq then 
		return false
	else return true end
end

function TrainerFu:UpdateNumberofLearnableSkills()
	local count = 0
	local cost = 0
	local countedSkills = {}
	for _,trainerType in pairs(self.db.char.trainerTypes) do
		for _,line in pairs(trainerType.lines) do
			for _,skillName in pairs(line.skills) do
				local skill = self.db.char.skillDB[skillName]
				if self:CanLearnSkill(skill) then
					count = count + 1 
					cost = cost + skill.cost
				end
			end
		end
	end
	self.costOfLearnableSkills = cost
	self.numberOfLearnambleSkills = count
end

function TrainerFu:CanLearnSkill(skill, characterLevelModifyer)
	local characterLevel = self.characterLevel
	if characterLevelModifyer then characterLevel = characterLevel + characterLevelModifyer end
	
	if skill.levelreq and skill.levelreq > characterLevel then 
		return false
	elseif skill.skillrankreq and self.skillRanks[skill.skillreq] and skill.skillrankreq > self.skillRanks[skill.skillreq].rank then
		return false
	elseif skill.skillrankreq and not self.skillRanks[skill.skillreq] then
		return false
	elseif skill.abilityreq and not skill.hasabilityreq then 
		return false
	else 
		return true 
	end
end

function TrainerFu:ToggleShowTrainingCost()
	self.db.profile.showtotaltrainingcost = not self.db.profile.showtotaltrainingcost
	self:Update()
end
function TrainerFu:IsShowTrainingCost()
	return self.db.profile.showtotaltrainingcost
end

function TrainerFu:RemoveTrainer(trainerName)
	local trainer = self.db.char.trainers[trainerName]
	self.db.char.trainerTypes[trainer.type].names[trainerName] = nil
	self.db.char.trainers[trainerName] = nil
	self:TriggerEvent("TrainerFu_SkillsUpdated")
end

function TrainerFu:RemoveTrainerType(trainerType)
	for trainerName,_ in pairs(self.db.char.trainerTypes[trainerType].names) do
		self:RemoveTrainer(trainerName)
	end
	self.db.char.trainerTypes[trainerType] = nil
	self:TriggerEvent("TrainerFu_SkillsUpdated")
end

function TrainerFu:RemoveTrainerOnClick(trainerIndex)
	--Remove trainer if ctrl+shift is down
	if IsControlKeyDown() and IsShiftKeyDown() then
		self:RemoveTrainer(trainerIndex)
	end
end

function TrainerFu:ToggleTrainerTypeExpanded(trainerType)
	--Remove trainerLine if ctrl+shift is down
	if IsControlKeyDown() and IsShiftKeyDown() then
		self:RemoveTrainerType(trainerType)
	else
		self.db.char.trainerTypes[trainerType].expanded = not self.db.char.trainerTypes[trainerType].expanded
	end
end

function TrainerFu:ToggleLineExpanded(trainerType, lineIndex)
	--Remove trainerLine if ctrl+shift is down
	if IsControlKeyDown() and IsShiftKeyDown() then
		table.remove(self.db.char.trainerTypes[trainerType]["lines"], lineIndex)
		self:TriggerEvent("TrainerFu_SkillsUpdated")
	else
		self.db.char.trainerTypes[trainerType]["lines"][lineIndex].expanded = not self.db.char.trainerTypes[trainerType]["lines"][lineIndex].expanded
	end
end

function TrainerFu:ClickOnSkill(skill, line, skillIndex) 
	if IsControlKeyDown() and IsShiftKeyDown() then
		table.remove(line.skills, skillIndex)
		self:TriggerEvent("TrainerFu_SkillsUpdated")
	else
		self:SkillInfoFrame(skill)
	end
end

function TrainerFu:OnTextUpdate()
	local skills = self:IsShowTrainableSkills() and self:GetNumberOfLearnableSkills() or ""
	local cost = ""
	if self:IsShowTrainingCost() and self.costOfLearnableSkills > 0 then
		cost = abacus:FormatMoneyShort(self.costOfLearnableSkills, true, coloured)
	end
	self:SetText( skills .." ".. cost )
end

function TrainerFu:OnTooltipUpdate()
	local cat = tablet:AddCategory()
	
	cat:AddLine(
		'text', L["Training cost: "].. self:GetTrainingCost()
	)
	cat:AddLine(
		'text', L["Trainable skills: "].. self:GetNumberOfLearnableSkills()
	)

	cat = tablet:AddCategory(
		'text', "Show all skills",
		'textR', 0, 'textG', 1, 'textB', 0,
		'showWithoutChildren', true,
		'func', 'ToggleShowNotLearnable', 'arg1', self,
		'hasCheck', true,
		'checked', self:IsShowingNotLearnable()
	)
	
	for typeName,trainerType in pairs(self.db.char.trainerTypes) do
		cat = tablet:AddCategory(
			'text', typeName,
			'columns', 2,
			'child_textR', 0, 'child_textG', 1,	'child_textB', 0,
			'child_justify2', "RIGHT",
			'child_indentation', 20,
			'showWithoutChildren', true,
			'hideBlankLine', true,
			'func', 'ToggleTrainerTypeExpanded', 'arg1', self, 'arg2', typeName,
			'hasCheck', true,
			'checked', true,
			'checkIcon', not trainerType.expanded and "Interface\\Buttons\\UI-PlusButton-Up" or "Interface\\Buttons\\UI-MinusButton-Up"
		)
	
		if trainerType.expanded then
			if self:IsShowingTrainerNames() then
				for _, trainer in pairs(self.db.char.trainers) do
					if trainer.type == typeName then 
						cat:AddLine(
							'text', trainer.name .." in ".. trainer.location .. self:GetCoordinateText(trainer),
							'func', 'RemoveTrainerOnClick', 'arg1', self, 'arg2', trainer.name,
							'indentation', 10,
							'noInherit', true
						)
					end
				end
			end
			
			for lineIndex,line in ipairs(trainerType.lines) do
				if table.getn(line.skills) ~= 0 then
					cat = tablet:AddCategory(
						'text', line.name,
						'columns', 2,
						'indentation', 10,
						'child_indentation', 25,
						'showWithoutChildren', true,
						'hideBlankLine', true,
						'func', 'ToggleLineExpanded', 'arg1', self, 'arg2', typeName, 'arg3', lineIndex,
						'hasCheck', true,
						'checked', true,
						'checkIcon', not line.expanded and "Interface\\Buttons\\UI-PlusButton-Up" or "Interface\\Buttons\\UI-MinusButton-Up"
					)
				end
				
				if line.expanded then
					for skillIndex,skillName in ipairs(line.skills) do
						skill = self.db.char.skillDB[skillName]
						local showSkill = false
						local skillNameColored = ""
						local skillLevelColored = nil
						local skillCost = nil
						local skillReq = ""
						local lvlString = " - Req: Lvl %d"
						local skillReqString = " - Req: %s %s"
						
						if self:CanLearnSkill(skill) then 
							skillNameColored = crayon:Green(skill.name)
							if self:IsShowingSkillLevelReq() and skill.levelreq > 0 then 
								skillLevelColored = crayon:Green( string.format(lvlString, skill.levelreq) )
							end
							showSkill = true
						elseif self:CanLearnSkillNext(skill) and self:IsShowingLearnableNext() then 
							skillNameColored = crayon:Yellow(skill.name)
							if self:IsShowingSkillLevelReq() and skill.levelreq > 0 then 
								skillLevelColored = crayon:Yellow( string.format(lvlString, skill.levelreq) )
							end
							showSkill = true
						elseif self:IsShowingNotLearnable() then
							skillNameColored = crayon:Red(skill.name)
							if self:IsShowingSkillLevelReq() and skill.levelreq > 0 then 
								skillLevelColored = crayon:Red( string.format(lvlString, skill.levelreq) )
							end
							showSkill = true
						end
						
						if skill.skillrankreq then
							if self:SkillReqIsMet(skill) then
								--skillReq = crayon:Green(skill.skillreq .." ".. skill.skillrankreq)
								skillReq = crayon:Green( string.format(skillReqString, skill.skillreq, skill.skillrankreq) )
							else
								skillReq = crayon:Red( string.format(skillReqString, skill.skillreq, skill.skillrankreq) )
							end
						end
						
						if self:IsShowingSkillCost() then
							skillCost = abacus:FormatMoneyFull(skill.cost, true, coloured)
						end
						
						if showSkill then
							cat:AddLine(
								'text', string.format(skillNameColored.."%s", skillLevelColored or skillReq or ""),
								'text2', skillCost,
								'hasCheck', true,
								'checked', true,
								'checkIcon', skill.icon,
								'func', 'ClickOnSkill', 'arg1', self, 'arg2', skill, 'arg3', line, 'arg4', skillIndex
							)
						end
					end
				end
			end
		end
	end
	
	tablet:SetTitle(self.characterName .." "..L["Level "].. self.characterLevel)
	tablet:SetHint(L["Click a skill to print information.\nCtrl-Shift-Click a trainer to remove it."])
end

function TrainerFu:SkillInfoFrame(skill)
	if self.skillInfoFrame then 
		self.skillInfoFrame:Hide()
		if self.skillInfoFrame.name == skill.name then
			self.skillInfoFrame = nil
			return
		end
	end	
	
	local levelReq
	local skillReq
	local abilityReq
	local skillName
	local skillDescription
	
	if self:CanLearnSkill(skill) then skillName = crayon:Green(skill.name)
	elseif self:CanLearnSkillNext(skill) then skillName = crayon:Yellow(skill.name)
	else skillName = crayon:Red(skill.name)
	end
	
	if skill.levelreq ~= 0 then 
		if skill.levelreq > self.characterLevel then
			levelReq = crayon:Red(L["Level "].. skill.levelreq)
		else
			levelReq = crayon:Green(L["Level "].. skill.levelreq)
		end
	else levelReq = "" end
	
	if skill.skillrankreq then
		if self:SkillReqIsMet(skill) then
			skillReq = crayon:Green(skill.skillreq .." ".. skill.skillrankreq)
		else
			skillReq = crayon:Red(skill.skillreq .." ".. skill.skillrankreq)
		end
	else skillReq = "" end
	
	if skill.abilityreq then 
		if self:SkillAbilityReqIsMet(skill) then
			abilityReq = ", ".. crayon:Green(skill.abilityreq)
		else
			abilityReq = ", ".. crayon:Red(skill.abilityreq)
		end
	else abilityReq = "" end
	
	if skill.description then skillDescription = "\n\n".. skill.description
	else skillDescription = "" end
	
	local cost = L["Cost: "].. abacus:FormatMoneyFull(skill.cost, true, coloured)
	local text = skillName .."\n".. L["Requires: "].. levelReq .." ".. skillReq .. abilityReq .."\n".. cost .. skillDescription
	
	local frame = CreateFrame("Frame", "TrainerFuSkillFrame", UIParent)
	frame:SetPoint(self.db.profile.skillinfoframepoint.point, self.db.profile.skillinfoframepoint.x, self.db.profile.skillinfoframepoint.y)
	frame:SetFrameStrata("TOOLTIP")
	frame:SetFrameLevel(99)
	frame:EnableMouse(true)
	frame:SetMovable(true)
	frame:SetClampedToScreen(true)
	frame:RegisterForDrag("LeftButton")
	frame:SetScript("OnDragStart", function()
		this:StartMoving()
	end)
	frame:SetScript("OnDragStop", function()
		this:StopMovingOrSizing()
		local point,relativeTo,relativePoint,xOfs,yOfs = this:GetPoint()
		self.db.profile.skillinfoframepoint.point = point
		self.db.profile.skillinfoframepoint.x = xOfs
		self.db.profile.skillinfoframepoint.y = yOfs
	end)

	frame:SetBackdrop({
		bgFile = "Interface\\ChatFrame\\ChatFrameBackground", tile = true, tileSize = 16,
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,
		insets = {left = 4, right = 4, top = 4, bottom = 4},
	})
	frame:SetBackdropBorderColor(.5, .5, .5, 1)
	frame:SetBackdropColor(0, 0, 0, .3)
	
	local skillIcon = frame:CreateTexture(nil, "ARTWORK")
	frame.skillIcon = skillIcon
	skillIcon:SetTexture(skill.icon)
	skillIcon:SetWidth(40)
	skillIcon:SetHeight(40)
	skillIcon:SetPoint("TOPRIGHT", frame, "TOPLEFT", 1, -3)
	
	
	local closeButton = CreateFrame("Button", "TrainerFuCloseButton", frame)
	closeButton:SetPoint("TOPRIGHT", frame, "TOPRIGHT", 0, 0)
	closeButton:SetWidth(30)
	closeButton:SetHeight(30)
	closeButton:SetNormalTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Up")
	closeButton:SetPushedTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Down")
	closeButton:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight")
	closeButton:SetScript("OnClick", function()
		frame:Hide()
	end)

	local skillText = frame:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	frame.skillText = skillText
	skillText:SetText(text)
	skillText:SetPoint("TOPLEFT", frame, "TOPLEFT", 10, -10)
	skillText:SetWidth(280)
	skillText:SetJustifyH("LEFT")
	skillText:SetJustifyV("TOP")

	frame.name = skill.name
	self.skillInfoFrame = frame
	frame:SetWidth(skillText:GetWidth() + 20)
	frame:SetHeight(skillText:GetHeight() + 20)
	frame:Show()
end
