TrainerFu_DemonTrainer = TrainerFu:NewModule("DemonTrainer", "AceEvent-2.0")

local gratuity = AceLibrary("Gratuity-2.0")
local BC = AceLibrary("Babble-Class-2.2")

function TrainerFu_DemonTrainer:OnInitialize()
	self.db = TrainerFu.db
end

function TrainerFu_DemonTrainer:OnEnable()
	if UnitClass("player") == BC["Warlock"] then
		self:RegisterEvent("MERCHANT_SHOW", "GetDemonTrainerSkills", 2)
		--Delay this some, may not work well with high latencies
		self:RegisterEvent("PET_BAR_UPDATE", function() self:ScheduleEvent(self.GetKnownDemonSpellRanks, 2, self) end, 2)
	end
end

function TrainerFu_DemonTrainer:GetDemonTrainerSkills()
	local trainer = TrainerFu_Trainer:GetTrainerInfo()

	--Return if its not a Demon Trainer
	if trainer.type ~= "Demon Trainer" then return end
	
	--Save trainer in trainer DB	
	self.db.char.trainers[trainer.name] = trainer
	
	local skillLines = {}
	skillLines[1] = {}
	skillLines[1].name = j
	skillLines[1].expanded = true
	skillLines[1].skills = {}
		
	local numberOfItems = GetMerchantNumItems() 
	for i=1, numberOfItems do
		local grimoareName, texture, price, quantity, numAvailable, isUsable = GetMerchantItemInfo(i);
		gratuity:SetMerchantItem(i)
		local descOrKnown = gratuity:GetLine(6)
		if descOrKnown ~= "Already known" and grimoareName then
			local skill = {}
			--Description
			skill.description = descOrKnown
			--Level requirement	
			local levelString = gratuity:GetLine(5)
			if levelString then
				local _,_,lvl = string.find(levelString, "%a*(%d+)")
				skill.levelreq = tonumber(lvl)
			else
				skill.levelreq = 0
			end
			--Full skill name
			local _,_,fullSkillName = string.find(grimoareName, "Grimoire%sof%s(.*)")
			skill.name = fullSkillName
			--Skill name without rank
			local _,_,skillName = string.find(fullSkillName, "([%s%w]+%w)%s?%(?")
			--Skill rank
			local _,_,skillRank = string.find(fullSkillName, "Rank%s(%d)")			
			--Icon path
			skill.icon = self.db.char.DemonSpellIcons[skillName] or texture
			--Cost
			skill.cost = price
			--Check for spell that is known and has no prereq
			if self.db.char.DemonSpellRanks[skillName] == 0 then
				skill = nil
			end
			
			if skill and skillRank then
				local rank = tonumber(skillRank)
				if self.db.char.DemonSpellRanks[skillName] and self.db.char.DemonSpellRanks[skillName] >= rank then
					skill = nil
				else
					rank = rank - 1
					if rank >= 1 then
						skill.abilityreq = skillName .." (Rank ".. rank ..")"
					end
					if self.db.char.DemonSpellRanks[skillName] and self.db.char.DemonSpellRanks[skillName] >= rank then
						skill.hasabilityreq = true
					end
				end
			end
			
			if skill then
				--Insert skill into skillDB
				self.db.char.skillDB[skill.name] = skill
				--Insert skill into skill line
				tinsert( skillLines[1].skills, skill.name)
			end
		end
	end
	
	--Save trainer in trainer type DB
	if not self.db.char.trainerTypes[trainer.type] then
		self.db.char.trainerTypes[trainer.type] = {}
		self.db.char.trainerTypes[trainer.type].expanded = true
		self.db.char.trainerTypes[trainer.type].names = {}
	end
	self.db.char.trainerTypes[trainer.type]["names"][trainer.name] = true
	self.db.char.trainerTypes[trainer.type].lines = skillLines
	
	self:TriggerEvent("TrainerFu_SkillsUpdated")
end

function TrainerFu_DemonTrainer:GetKnownDemonSpellRanks()
	if not self.db.char.DemonSpellRanks then self.db.char.DemonSpellRanks = {} end
	if not self.db.char.DemonSpellIcons then self.db.char.DemonSpellIcons = {} end
	numSpells, petToken = HasPetSpells();
	if numSpells ~= nil and petToken == "DEMON" then
		for i=1, numSpells do
			local spellName, spellRank = GetSpellName(i, BOOKTYPE_PET)
			if spellRank ~= "" then
				local _,_,spellRank = string.find(spellRank, "Rank%s(%d)")
				self.db.char.DemonSpellRanks[spellName] = tonumber(spellRank)
			elseif spellName then
				self.db.char.DemonSpellRanks[spellName] = 0
			end
			spellIcon = GetSpellTexture(i, BOOKTYPE_PET)
			if spellIcon then
				self.db.char.DemonSpellIcons[spellName] = spellIcon
			end
		end
	end
end
