local L = AceLibrary("AceLocale-2.2"):new("TrainerFu-Trainer")

local gratuity = AceLibrary("Gratuity-2.0")

TrainerFu_Trainer = TrainerFu:NewModule("Trainer", "AceEvent-2.0")

function TrainerFu_Trainer:OnInitialize()
	self.db = TrainerFu.db
end

function TrainerFu_Trainer:OnEnable()
	self:RegisterEvent("TRAINER_SHOW", "VisitingTrainer", 2)
	self:RegisterEvent("TRAINER_CLOSED", "GetTrainerSkills", 2)
end

function TrainerFu_Trainer:VisitingTrainer()
	self.trainerInfo = self:GetTrainerInfo()
end

function TrainerFu_Trainer:GetTrainerSkills()
	--Don't save profession trainer info if profession not known
	if IsTradeskillTrainer() then
		local tradeType = GetTrainerServiceSkillLine(2)
		if not TrainerFu.skillRanks[tradeType] then
			return
		end
	end
	
	local trainer = self.trainerInfo
	
	--Expand all categories
	ExpandTrainerSkillLine(0);
	
	--Need to set available and unavailable visible, we do not want already learned
	SetTrainerServiceTypeFilter("available", 1);
	SetTrainerServiceTypeFilter("unavailable", 1);
	SetTrainerServiceTypeFilter("used", 0);
	
	--Save trainer in trainer DB	
	self.db.char.trainers[trainer.name] = trainer
	
	skillLines = self:GetSkillInformation()
	
	--Save trainer in trainer type DB
	if not self.db.char.trainerTypes[trainer.type] then
		self.db.char.trainerTypes[trainer.type] = {}
		self.db.char.trainerTypes[trainer.type].expanded = true
		self.db.char.trainerTypes[trainer.type].names = {}
	end
	self.db.char.trainerTypes[trainer.type]["names"][trainer.name] = true
	self.db.char.trainerTypes[trainer.type].lines = skillLines
	
	self:TriggerEvent("TrainerFu_SkillsUpdated")
end

function TrainerFu_Trainer:GetSkillInformation()
	local lines
	--Stupid hack cus GetTrainerSkillLines() return nil if trainer only got one skill line...
	if not GetTrainerSkillLines() then
		lines = { GetTrainerServiceSkillLine(2) }
	else
		lines = { GetTrainerSkillLines() }
	end	

	local lineIndex = {}	
	local skillLines = {}
	--Init all skill lines
	for i,j in pairs(lines) do
		lineIndex[j] = i
		skillLines[i] = {}
		skillLines[i].name = j
		skillLines[i].expanded = true
		skillLines[i].skills = {}
	end
	
	--Get skill information
	local numTrainerServices = GetNumTrainerServices()
	for i=1, numTrainerServices do
		local serviceName, serviceSubText, serviceType, isExpanded = GetTrainerServiceInfo(i);
		
		if not serviceName or not serviceType then
			return;
		end
		
		if serviceType ~= "header" then
			local skill = {}
			--Skill name
			local skillName = serviceName
			if serviceSubText ~= "" then skillName = skillName.." ("..serviceSubText..")" end
			skill.name = skillName		
			--Icon path
			skill.icon = GetTrainerServiceIcon(i)
			--Cost
			skill.cost = GetTrainerServiceCost(i)
			--Level requirement
			skill.levelreq = GetTrainerServiceLevelReq(i)
			--Description
			local description = GetTrainerServiceDescription(i)
			if description then
				--Get info if normal skill
				skill.description = description
			else
				--Get info from tooltip if profession
				gratuity:SetTrainerService(i)
				local description = gratuity:GetLine(3)
				if description then 
					skill.description = description
				end
			end
			--Required skillrank
			local skillreq, skillrankreq, hasReq = GetTrainerServiceSkillReq(i);
			if skillrankreq > 0 then 
				skill.skillreq = skillreq
				skill.skillrankreq = skillrankreq
			end
			
			local numRequirements = GetTrainerServiceNumAbilityReq(i);
			if numRequirements > 0 then
				local abilityReq, hasAbilityReq = GetTrainerServiceAbilityReq(i, 1)
				skill.abilityreq = abilityReq
				skill.hasabilityreq = hasAbilityReq
			end
			--Insert skill into skillDB
			self.db.char.skillDB[skillName] = skill
			--Insert skill into skill line
			local index = lineIndex[GetTrainerServiceSkillLine(i)]
			tinsert( skillLines[index].skills, skillName)
		end
	end
	return skillLines
end
	

function TrainerFu_Trainer:GetTrainerInfo()
	--Get coordinates
	local x, y = GetPlayerMapPosition("player")

	--Get trainer name and type
	gratuity:SetUnit("npc")
	local trainerName =  gratuity:GetLine(1)
	local trainerType = gratuity:GetLine(2)
	
	local trainer = {}
	trainer.name = trainerName or ""
	trainer.type = trainerType or ""
	trainer.location = GetRealZoneText()
	trainer.xcoord = x
	trainer.ycoord = y
	
	return trainer
end
